@@ -5,7 +5,7 @@
 TODO
 """
 
-__version__ = '0.9.93'
+__version__ = '0.9.94'
 
 from .action import \
     Action, \
@@ -17,7 +17,7 @@ from .app import \
     Context
 
 from .doc import \
-    get_doc_requests
+    create_doc_requests
 
 from .model import \
     ActionModel, \
